%% Look for a phase transition in problem solutions

%%
addpath('../misc/gaimc'); % get bfs function
addpath('~/devextern/Clp-1.9.0/');
addpath('../../matlab');


%%

nrep = 10;

k = 20;
q = 2;
d = 1;
pns = [linspace(0.5,8,13) 10 12.5 15];


%%
alpha = 0;
beta = 1;
fvals = zeros(nrep,length(pns));
rvals = zeros(nrep,length(pns));
ncorr = zeros(nrep,length(pns));

fvals_ref = zeros(nrep, length(pns));

rvals_iso = zeros(nrep,length(pns));
ncorr_iso = zeros(nrep,length(pns));

rvals_bp = zeros(nrep,length(pns));
ncorr_bp = zeros(nrep,length(pns));

rvals_llp = zeros(nrep,length(pns));
ncorr_llp = zeros(nrep,length(pns));
fvals_llp = zeros(nrep,length(pns));


for pi = 1:length(pns)
    p = pns(pi)./k^2;
    for ri = 1:nrep
        [A,B,L] = align_grid_test_data(k,q,p,d);
        L = spones(L); % ignore weights for this test
        [S,w,li,lj] = netalign_setup(A,B,L);
        
        
        [xmr status] = netalignmr(S,w,alpha,beta,li,lj,0.4,25,[],5000);
        
        [ma mb mi weight overlap] = mwmround(xmr,S,w,li,lj);
        rvals(ri,pi) = alpha*weight + beta*overlap;
        ncorr(ri,pi) = sum(ma==mb);        
        fvals(ri,pi) = status(3);
        rv = rvals(ri,pi);
        nc = ncorr(ri,pi);
        
        [ma mb mi weight overlap] = mwmround(li==lj,S,w,li,lj);
        fvals_ref(ri,pi) = alpha*weight + beta*overlap;
        
        xbp = netalignbp(S,w,1,2,li,lj);
        [ma mb mi weight overlap] = mwmround(xbp,S,w,li,lj);
        rvals_bp(ri,pi) = alpha*weight + beta*overlap;
        ncorr_bp(ri,pi) = sum(ma==mb);
        
        xiso = isorank(S,w,1,19,li,lj);
        [ma mb mi weight overlap] = mwmround(xiso,S,w,li,lj);
        rvals_iso(ri,pi) = alpha*weight + beta*overlap;
        ncorr_iso(ri,pi) = sum(ma==mb);

        [xllp fupper] = netalign_lagrange(S,w,alpha,beta,li,lj);
        [ma mb mi weight overlap] = mwmround(xllp,S,w,li,lj);
        rvals_llp(ri,pi) = alpha*weight + beta*overlap;
        ncorr_llp(ri,pi) = sum(ma==mb);
        fvals_llp(ri,pi) = fupper;
        
        disp([pi pns(pi) ri status(3) rv nc ...
            rvals_bp(ri,pi) ncorr_bp(ri,pi) ...
            rvals_iso(ri,pi) ncorr_iso(ri,pi) ...
            rvals_llp(ri,pi) ncorr_llp(ri,pi) fvals_llp(ri,pi)])
    end
end
save(sprintf('results-k-%i.mat',k), 'pns','fvals','fvals_ref','rvals','ncorr','*_bp','*_llp','*_iso')
return

%%

nrep = 10;

k = 35;
q = 1;
pns = linspace(0.5,15,30);


%%
alpha = 0;
beta = 1;
fvals = zeros(nrep,length(pns));
rvals = zeros(nrep,length(pns));
ncorr = zeros(nrep,length(pns));

for pi = 1:length(pns)
    p = pns(pi)./k^2;
    for ri = 1:nrep
        [A,B,L] = align_grid_test_data(k,q,p);
        [S,w,li,lj] = netalign_setup(A,B,L);
        [f,C,b,Si,Sj] = netalign_lp_prob(S,w,alpha,beta,li,lj);
        y = clp([],-f,C,b,[],[],zeros(size(f)),ones(size(f)));
        xf = y(1:length(w)); 
        Sf = sparse(Si,Sj,y(length(w)+1:end),size(S,1),size(S,2)); 
        Sf = max(Sf,Sf');
        
        fvals(ri,pi) = f'*y;
        
        
        [ma mb mi weight overlap] = mwmround(xf,S,w,li,lj);
        rv = alpha*weight + beta*overlap;
        nc = sum(ma==mb);
        [ma mb mi weight overlap] = mwmround(S*xf,S,w,li,lj);
        if alpha*weight + beta*overlap > rv
            rv = alpha*weight + beta*overlap;
            nc = sum(ma==mb);
        end
        [ma mb mi weight overlap] = mwmround(Sf*xf,S,w,li,lj);
        if alpha*weight + beta*overlap > rv
            rv = alpha*weight + beta*overlap;
            nc = sum(ma==mb);
        end
        
        rvals(ri,pi) = rv;
        ncorr(ri,pi) = nc;
        
        disp([pi pns(pi) ri f'*y rv nc])

    end
end
save 'results-k-35.mat' pns fvals rvals ncorr

%%
alpha = 0;
beta = 1;
fvals = zeros(nrep,length(pns));
rvals = zeros(nrep,length(pns));
ncorr = zeros(nrep,length(pns));

matlabpool(3);

for pi = 21:length(pns)
    p = pns(pi)./k^2;
    parfor ri = 1:nrep
        [A,B,L] = align_grid_test_data(k,q,p);
        [S,w,li,lj] = netalign_setup(A,B,L);
        [f,C,b,Si,Sj] = netalign_lp_prob(S,w,alpha,beta,li,lj);
        y = clp([],-f,C,b,[],[],zeros(size(f)),ones(size(f)));
        xf = y(1:length(w)); 
        Sf = sparse(Si,Sj,y(length(w)+1:end),size(S,1),size(S,2)); 
        Sf = max(Sf,Sf');
        
        fvals(ri,pi) = f'*y;
        
        
        [ma mb mi weight overlap] = mwmround(xf,S,w,li,lj);
        rv = alpha*weight + beta*overlap;
        nc = sum(ma==mb);
        [ma mb mi weight overlap] = mwmround(S*xf,S,w,li,lj);
        if alpha*weight + beta*overlap > rv
            rv = alpha*weight + beta*overlap;
            nc = sum(ma==mb);
        end
        [ma mb mi weight overlap] = mwmround(Sf*xf,S,w,li,lj);
        if alpha*weight + beta*overlap > rv
            rv = alpha*weight + beta*overlap;
            nc = sum(ma==mb);
        end
        
        rvals(ri,pi) = rv;
        ncorr(ri,pi) = nc;
        
        disp([pi pns(pi) ri f'*y rv nc])

    end
end
save 'results-k-35-2.mat' pns fvals rvals ncorr


%%
plot(pns, mean(fvals), '--', pns, mean(rvals), '.-', pns, mean(ncorr), '*-.');
